@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2010-2012  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2010-2013  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -648,7 +648,9 @@ StreamDecoder::~StreamDecoder()
 }
 
 #ifdef ENABLE_LIBAVCODEC
-FFMpegStreamDecoder::FFMpegStreamDecoder(std::istream& s):audioFound(false),videoFound(false),stream(s),formatCtx(NULL),avioContext(NULL)
+FFMpegStreamDecoder::FFMpegStreamDecoder(std::istream& s)
+ : audioFound(false),videoFound(false),stream(s),formatCtx(NULL),audioIndex(-1),
+   videoIndex(-1),customAudioDecoder(NULL),customVideoDecoder(NULL),avioContext(NULL)
 {
 	valid=false;
 #ifdef HAVE_AVIO_ALLOC_CONTEXT
@@ -709,7 +711,7 @@ FFMpegStreamDecoder::FFMpegStreamDecoder(std::istream& s):audioFound(false),vide
 			videoFound=true;
 			videoIndex=(int32_t)i;
 		}
-		else if(formatCtx->streams[i]->codec->codec_type==AVMEDIA_TYPE_AUDIO && audioFound==false)
+		else if(formatCtx->streams[i]->codec->codec_type==AVMEDIA_TYPE_AUDIO && formatCtx->streams[i]->codec->codec_id!=CODEC_ID_NONE && audioFound==false)
 		{
 			audioFound=true;
 			audioIndex=(int32_t)i;
@@ -766,10 +768,16 @@ bool FFMpegStreamDecoder::decodeNextFrame()
 	//Should use dts
 	uint32_t mtime=pkt.dts*1000*time_base.num/time_base.den;
 
-	if(pkt.stream_index==(int)audioIndex)
-		customAudioDecoder->decodePacket(&pkt, mtime);
-	else
-		customVideoDecoder->decodePacket(&pkt, mtime);
+	if (pkt.stream_index==(int)audioIndex)
+	{
+		if (customAudioDecoder)
+			customAudioDecoder->decodePacket(&pkt, mtime);
+	}
+	else 
+	{
+		if (customVideoDecoder)
+			customVideoDecoder->decodePacket(&pkt, mtime);
+	}
 	av_free_packet(&pkt);
 	return true;
 }
