@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2013  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -586,7 +586,7 @@ ASObject* ABCVm::constructFunction(call_context* th, IFunction* f, ASObject** ar
 {
 	//See ECMA 13.2.2
 	if(f->inClass)
-		throw Class<TypeError>::getInstanceS("Error #1064: Cannot call method as constructor");
+		throwError<TypeError>(kCannotCallMethodAsConstructor, "");
 
 	assert(f->is<SyntheticFunction>());
 	SyntheticFunction* sf=f->as<SyntheticFunction>();
@@ -672,7 +672,7 @@ void ABCVm::construct(call_context* th, int m)
 
 		default:
 		{
-			throw Class<TypeError>::getInstanceS("Error #1007: Instantiation attempted on a non-constructor");
+			throwError<TypeError>(kConstructOfNonFunctionError);
 		}
 	}
 
@@ -685,7 +685,7 @@ void ABCVm::constructGenericType(call_context* th, int m)
 {
 	LOG(LOG_CALLS, _("constructGenericType ") << m);
 	if (m != 1)
-			throw Class<TypeError>::getInstanceS("Error #1128");
+		throwError<TypeError>(kWrongTypeArgCountError, "function", "1", Integer::toString(m));
 	ASObject** args=g_newa(ASObject*, m);
 	for(int i=0;i<m;i++)
 		args[m-i-1]=th->runtime_stack_pop();
@@ -720,6 +720,16 @@ void ABCVm::constructGenericType(call_context* th, int m)
 
 	Class_base* o_class = o_template->applyType(t);
 
+	// Register the type name in the global scope. The type name
+	// is later used by the coerce opcode.
+	_R<ASObject> global = th->scope_stack[0].object;
+	QName qname = o_class->class_name;
+	if (!global->hasPropertyByMultiname(qname, false, false))
+	{
+		o_class->incRef();
+		global->setVariableByQName(qname.name,nsNameAndKind(qname.ns,NAMESPACE),o_class,DECLARED_TRAIT);
+	}
+
 	for(int i=0;i<m;++i)
 		args[i]->decRef();
 
@@ -1517,15 +1527,15 @@ bool ABCVm::isTypelate(ASObject* type, ASObject* obj)
 		case T_STRING:
 			obj->decRef();
 			type->decRef();
-			throw Class<TypeError>::getInstanceS("Error #1009");
+			throwError<TypeError>(kConvertNullToObjectError);
 		case T_UNDEFINED:
 			obj->decRef();
 			type->decRef();
-			throw Class<TypeError>::getInstanceS("Error #1010");
+			throwError<TypeError>(kConvertUndefinedToObjectError);
 		case T_CLASS:
 			break;
 		default:
-			throw Class<TypeError>::getInstanceS("Error #1041");
+			throwError<TypeError>(kIsTypeMustBeClassError);
 	}
 
 	c=static_cast<Class_base*>(type);
@@ -1599,7 +1609,7 @@ ASObject* ABCVm::asTypelate(ASObject* type, ASObject* obj)
 	{
 		obj->decRef();
 		type->decRef();
-		throw Class<TypeError>::getInstanceS("Error #1009: Not a type in asTypelate");
+		throwError<TypeError>(kConvertNullToObjectError);
 	}
 	Class_base* c=static_cast<Class_base*>(type);
 	//Special case numeric types
@@ -1714,16 +1724,17 @@ void ABCVm::constructProp(call_context* th, int n, int m)
 	ASObject* obj=th->runtime_stack_pop();
 
 	_NR<ASObject> o=obj->getVariableByMultiname(*name);
-	name->resetNameIfObject();
 
 	if(o.isNull())
 	{
 		for(int i=0;i<m;++i)
 			args[i]->decRef();
 		obj->decRef();
-		throw Class<ReferenceError>::getInstanceS("Error #1065: Variable is not defined.");
+		throwError<ReferenceError>(kUndefinedVarError, name->normalizedName());
 	}
 
+	name->resetNameIfObject();
+
 	LOG(LOG_CALLS,_("Constructing"));
 	ASObject* ret;
 	try
@@ -1738,7 +1749,7 @@ void ABCVm::constructProp(call_context* th, int n, int m)
 			ret = constructFunction(th, o->as<IFunction>(), args, m);
 		}
 		else
-			throw Class<TypeError>::getInstanceS("Error #1007: Cannot construct such an object in constructProp");
+			throwError<TypeError>(kConstructOfNonFunctionError);
 	}
 	catch(ASObject* exc)
 	{
@@ -1851,15 +1862,17 @@ void ABCVm::getDescendants(call_context* th, int n)
 			}
 			else
 			{
+				tiny_string objName = obj->getClassName();
 				obj->decRef();
-				throw Class<TypeError>::getInstanceS("Only XML and XMLList objects have descendants");
+				throwError<TypeError>(kDescendentsError, objName);
 			}
 		}
 	}
 	else
 	{
+		tiny_string objName = obj->getClassName();
 		obj->decRef();
-		throw Class<TypeError>::getInstanceS("Only XML and XMLList objects have descendants");
+		throwError<TypeError>(kDescendentsError, objName);
 	}
 	XMLList* retObj=Class<XMLList>::getInstanceS(ret);
 	th->runtime_stack_push(retObj);
@@ -2183,7 +2196,7 @@ void ABCVm::callImpl(call_context* th, ASObject* f, ASObject* obj, ASObject** ar
 		else
 		{
 			f->decRef();
-			throw Class<TypeError>::getInstanceS("Error #1006: Tried to call something that is not a function");
+			throwError<TypeError>(kCallOfNonFunctionError, "Object");
 		}
 	}
 	LOG(LOG_CALLS,_("End of call ") << m << ' ' << f);
@@ -2294,12 +2307,15 @@ bool ABCVm::instanceOf(ASObject* value, ASObject* type)
 	}
 
 	if(!type->is<Class_base>())
-		throw Class<TypeError>::getInstanceS("Error #1040: instanceOf expects a class of function as second parameter!");
+		throwError<TypeError>(kCantUseInstanceofOnNonObjectError);
 
 	if(value->is<Class_base>())
-		return value->as<Class_base>()->isSubClass(type->as<Class_base>());
+		// Classes are instance of Class and Object but not
+		// itself or super classes
+		return type == Class_object::getClass() || 
+			type == Class<ASObject>::getClass();
 	else
-		return value->getClass() && value->getClass()->isSubClass(type->as<Class_base>());
+		return value->getClass() && value->getClass()->isSubClass(type->as<Class_base>(), false);
 }
 
 Namespace* ABCVm::pushNamespace(call_context* th, int n)
