@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2011-2012  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2011-2013  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -50,6 +50,7 @@ public:
 	static ASObject* generator(TemplatedClass<Vector>* o_class, ASObject* const* args, const unsigned int argslen);
 
 	void setTypes(const std::vector<Type*>& types);
+	bool sameType(const std::vector<Type*>& types) const;
 
 	//Overloads
 	tiny_string toString(bool debugMsg=false);
@@ -71,6 +72,10 @@ public:
                 return vec.at(index);
         }
 
+	//Appends an object to the Vector. o is coerced to vec_type.
+	//Takes ownership of o.
+	void append(ASObject *o);
+
 	//TODO: do we need to implement generator?
 	ASFUNCTION(_constructor);
 	ASFUNCTION(_applytype);
@@ -101,7 +106,5 @@ public:
 	ASFUNCTION(some);
 };
 
-
-
 }
 #endif /* SCRIPTING_TOPLEVEL_VECTOR_H */
