@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2013  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -79,6 +79,7 @@ void XML::sinit(Class_base* c)
 	c->setDeclaredMethodByQName("localName",AS3,Class<IFunction>::getFunction(localName),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("name",AS3,Class<IFunction>::getFunction(name),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("namespace",AS3,Class<IFunction>::getFunction(_namespace),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("normalize",AS3,Class<IFunction>::getFunction(_normalize),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("descendants",AS3,Class<IFunction>::getFunction(descendants),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("appendChild",AS3,Class<IFunction>::getFunction(appendChild),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("parent",AS3,Class<IFunction>::getFunction(parent),NORMAL_METHOD,true);
@@ -628,8 +629,8 @@ ASFUNCTIONBODY(XML,inScopeNamespaces)
 ASFUNCTIONBODY(XML,addNamespace)
 {
 	XML *th = obj->as<XML>();
-	if(argslen == 0)
-		throw Class<ArgumentError>::getInstanceS("Error #1063: Non-optional argument missing");
+	_NR<ASObject> newNamespace;
+	ARG_UNPACK(newNamespace);
 
 	xmlpp::Element *element=dynamic_cast<xmlpp::Element*>(th->node);
 	if(!element)
@@ -637,7 +638,7 @@ ASFUNCTIONBODY(XML,addNamespace)
 
 	// TODO: check if the prefix already exists
 
-	Namespace *ns=dynamic_cast<Namespace *>(args[0]);
+	Namespace *ns=dynamic_cast<Namespace *>(newNamespace.getPtr());
 	if(ns)
 	{
 		tiny_string uri=ns->getURI();
@@ -647,7 +648,7 @@ ASFUNCTIONBODY(XML,addNamespace)
 	}
 	else
 	{
-		tiny_string uri=args[0]->toString();
+		tiny_string uri=newNamespace->toString();
 		element->set_namespace_declaration(uri);
 	}
 
@@ -739,21 +740,21 @@ ASFUNCTIONBODY(XML,_namespace)
 ASFUNCTIONBODY(XML,_setLocalName)
 {
 	XML *th = obj->as<XML>();
-	if(argslen == 0)
-		throw Class<ArgumentError>::getInstanceS("Error #1063: Non-optional argument missing");
+	_NR<ASObject> newName;
+	ARG_UNPACK(newName);
 
 	xmlElementType nodetype=th->node->cobj()->type;
 	if(nodetype==XML_TEXT_NODE || nodetype==XML_COMMENT_NODE)
 		return NULL;
 
 	tiny_string new_name;
-	if(args[0]->is<ASQName>())
+	if(newName->is<ASQName>())
 	{
-		new_name=args[0]->as<ASQName>()->getLocalName();
+		new_name=newName->as<ASQName>()->getLocalName();
 	}
 	else
 	{
-		new_name=args[0]->toString();
+		new_name=newName->toString();
 	}
 
 	th->setLocalName(new_name);
@@ -765,7 +766,7 @@ void XML::setLocalName(const tiny_string& new_name)
 {
 	if(!isXMLName(Class<ASString>::getInstanceS(new_name)))
 	{
-		throw Class<TypeError>::getInstanceS("Error #1117");
+		throwError<TypeError>(kXMLInvalidName, new_name);
 	}
 
 	node->set_name(new_name);
@@ -774,8 +775,8 @@ void XML::setLocalName(const tiny_string& new_name)
 ASFUNCTIONBODY(XML,_setName)
 {
 	XML *th = obj->as<XML>();
-	if(argslen == 0)
-		throw Class<ArgumentError>::getInstanceS("Error #1063: Non-optional argument missing");
+	_NR<ASObject> newName;
+	ARG_UNPACK(newName);
 
 	xmlElementType nodetype=th->node->cobj()->type;
 	if(nodetype==XML_TEXT_NODE || nodetype==XML_COMMENT_NODE)
@@ -783,15 +784,15 @@ ASFUNCTIONBODY(XML,_setName)
 
 	tiny_string localname;
 	tiny_string ns_uri;
-	if(args[0]->is<ASQName>())
+	if(newName->is<ASQName>())
 	{
-		ASQName *qname=args[0]->as<ASQName>();
+		ASQName *qname=newName->as<ASQName>();
 		localname=qname->getLocalName();
 		ns_uri=qname->getURI();
 	}
-	else if (!args[0]->is<Undefined>())
+	else if (!newName->is<Undefined>())
 	{
-		localname=args[0]->toString();
+		localname=newName->toString();
 	}
 
 	th->setLocalName(localname);
@@ -803,8 +804,8 @@ ASFUNCTIONBODY(XML,_setName)
 ASFUNCTIONBODY(XML,_setNamespace)
 {
 	XML *th = obj->as<XML>();
-	if(argslen == 0)
-		throw Class<ArgumentError>::getInstanceS("Error #1063: Non-optional argument missing");
+	_NR<ASObject> newNamespace;
+	ARG_UNPACK(newNamespace);
 
 	xmlElementType nodetype=th->node->cobj()->type;
 	if(nodetype==XML_TEXT_NODE ||
@@ -814,21 +815,21 @@ ASFUNCTIONBODY(XML,_setNamespace)
 
 	tiny_string ns_uri;
 	tiny_string ns_prefix;
-	if(args[0]->is<Namespace>())
+	if(newNamespace->is<Namespace>())
 	{
-		Namespace *ns=args[0]->as<Namespace>();
+		Namespace *ns=newNamespace->as<Namespace>();
 		ns_uri=ns->getURI();
 		bool prefix_is_undefined=true;
 		ns_prefix=ns->getPrefix(prefix_is_undefined);
 	}
-	else if(args[0]->is<ASQName>())
+	else if(newNamespace->is<ASQName>())
 	{
-		ASQName *qname=args[0]->as<ASQName>();
+		ASQName *qname=newNamespace->as<ASQName>();
 		ns_uri=qname->getURI();
 	}
-	else if (!args[0]->is<Undefined>())
+	else if (!newNamespace->is<Undefined>())
 	{
-		ns_uri=args[0]->toString();
+		ns_uri=newNamespace->toString();
 	}
 
 	th->setNamespace(ns_uri, ns_prefix);
@@ -904,6 +905,48 @@ ASFUNCTIONBODY(XML,_setChildren)
 	return th;
 }
 
+ASFUNCTIONBODY(XML,_normalize)
+{
+	XML* th=obj->as<XML>();
+	th->normalize();
+
+	th->incRef();
+	return th;
+}
+
+void XML::normalize()
+{
+	normalizeRecursive(node);
+}
+
+void XML::normalizeRecursive(xmlpp::Node *node)
+{
+	// TODO: merge adjacent text nodes
+
+	xmlpp::Node::NodeList children=node->get_children();
+	xmlpp::Node::NodeList::const_iterator it=children.begin();
+	for(;it!=children.end();++it)
+	{
+		if (dynamic_cast<xmlpp::Element*>(*it))
+		{
+			normalizeRecursive(*it);
+		}
+		else
+		{
+			xmlpp::TextNode *textnode = dynamic_cast<xmlpp::TextNode*>(*it);
+			if (textnode && textnode->is_white_space())
+				node->remove_child(*it);
+		}
+	}
+}
+
+void XML::addTextContent(const tiny_string& str)
+{
+	assert(getNodeKind() == XML_TEXT_NODE);
+
+	xmlNodeAddContentLen(node->cobj(), BAD_CAST str.raw_buf(), str.numBytes());
+}
+
 void XML::removeAllChildren()
 {
 	xmlpp::Node::NodeList children=node->get_children();
